/*
 * Copyright (c) 1998-2008 TeamDev Ltd. All Rights Reserved. Use is subject to
 * license terms.
 */
package com.teamdev.xpcom.mozilla.demo;

import static com.teamdev.xpcom.mozilla.demo.components.LocationCombo.MyEditor.STATE_END_LOADING;
import static com.teamdev.xpcom.mozilla.demo.components.LocationCombo.MyEditor.STATE_TAB_CHANGED;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.jar.JarFile;

import javax.imageio.ImageIO;
import javax.naming.OperationNotSupportedException;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.DefaultEditorKit;

import org.jdesktop.layout.GroupLayout;
import org.mozilla.interfaces.nsIWebBrowserChrome;
import org.mozilla.xpcom.XPCOMException;
import org.w3c.dom.html.HTMLImageElement;

import teamdev.jxdesktop.PlatformContext;

import com.jniwrapper.util.Logger;
import com.teamdev.jxbrowser.DownloadManager;
import com.teamdev.jxbrowser.DownloadManagerFactory;
import com.teamdev.jxbrowser.WebBrowser;
import com.teamdev.jxbrowser.WebBrowserFactory;
import com.teamdev.jxbrowser.WebBrowserWindow;
import com.teamdev.jxbrowser.WindowCreator;
import com.teamdev.jxbrowser.console.ConsoleListener;
import com.teamdev.jxbrowser.console.MessageEvent;
import com.teamdev.jxbrowser.console.ScriptErrorEvent;
import com.teamdev.jxbrowser.event.ContextMenuEvent;
import com.teamdev.jxbrowser.event.ContextMenuListener;
import com.teamdev.jxbrowser.event.LocationEvent;
import com.teamdev.jxbrowser.event.ProgressEvent;
import com.teamdev.jxbrowser.event.RequestAdapter;
import com.teamdev.jxbrowser.event.RequestListener;
import com.teamdev.jxbrowser.event.SecurityEvent;
import com.teamdev.jxbrowser.event.StateEvent;
import com.teamdev.jxbrowser.event.StatusChangeEvent;
import com.teamdev.jxbrowser.event.StatusChangeListener;
import com.teamdev.jxbrowser.event.StatusEvent;
import com.teamdev.jxbrowser.printing.WebBrowserPrinting;
import com.teamdev.xpcom.AsyncHandler;
import com.teamdev.xpcom.AsyncHandlerFactory;
import com.teamdev.xpcom.DefaultAsyncHandler;
import com.teamdev.xpcom.Xpcom;
import com.teamdev.xpcom.mozilla.demo.components.AboutDialog;
import com.teamdev.xpcom.mozilla.demo.components.ActionButton;
import com.teamdev.xpcom.mozilla.demo.components.LineBevel;
import com.teamdev.xpcom.mozilla.demo.components.LocationCombo;
import com.teamdev.xpcom.mozilla.demo.components.OptionsDialog;
import com.teamdev.xpcom.mozilla.demo.components.PrintDialog;
import com.teamdev.xpcom.mozilla.demo.components.PrintPreviewPanel;
import com.teamdev.xpcom.mozilla.demo.components.ProgressDialog;
import com.teamdev.xpcom.mozilla.demo.components.PropertiesDialog;
import com.teamdev.xpcom.mozilla.demo.components.SaveJFileChooser;
import com.teamdev.xpcom.mozilla.demo.components.SourceFrame;

/**
 * @author Ikryanov Vladimir
 */
@SuppressWarnings("serial")
public class JxBrowserDemo
{

    private static int MENU_ACSELERATOR_KEY = Toolkit.getDefaultToolkit()
            .getMenuShortcutKeyMask();

    public static final String _aboutBlank = "about:blank";

    private static final Logger LOG = Logger.getInstance(JxBrowserDemo.class);

    private static final String PATH_SEPARATOR = System
            .getProperty("path.separator");

    // UI components
    private final JPanel _statusPanel = new JPanel();

    private final JLabel _statusText = new JLabel();

    private LocationCombo _locationBox;

    private TabbedBrowsersPane _browsersPane;

    private JProgressBar _progressBar;

    private LoaderImage _loaderImage;

    // Browser actions
    private CloseCurrentBrowserAction _closeCurrentBrowserAction;

    private CreateNewBrowserTabAction _createNewBrowserTabAction;

    private CutAction _cutAction;

    private CopyAction _copyAction;

    private PasteAction _pasteAction;

    private DeleteAction _deleteAction;

    private GoBackAction _goBackAction;

    private GoForwardAction _goForwardAction;

    private RefreshAction _refreshAction;

    private GoHomeAction _homeAction;

    private StopAction _stopAction;

    private SourceAction _sourceAction;

    private PageSetupAction _pageSetup;

    private PrintPreviewAction _printPreview;

    private PrintAction _print;

    private SaveAction _save;

    private OpenAction _open;

    private ShowDownloadsAction _showDownloads;

    private SaveAsImageAction _saveAsImage;

    // Browser utils
    private MyContextMenuListener _myContextMenuListener;

    private final WindowCreator _childCreator;

    private final ConsoleListener listener = new ConsoleListener()
    {
        public void actionPerformed(MessageEvent event)
        {
            if (event instanceof ScriptErrorEvent)
            {
                // you'r On script error reaction
            }
            System.err.println(event.getMessage());
        }
    };

    {
        // ConsoleService service = ConsoleService.getConsoleService();
        // uncomment the following line to enable console messages output
        // service.registerConsoleListener(listener);
    }

    private boolean _HasFocusInLocationCombo;
    private CardLayout printPreviewLayout;
    private JPanel contentPanel;
    private JPanel printPreviewPanel;

    private static void setupLF()
    {
        try
        {
            // never use the system look and fill at GNOME with GTK,
            // never use Mac style menus on MAC OS X
            if (!Xpcom.isLinux())
            {
                UIManager.setLookAndFeel(UIManager
                        .getSystemLookAndFeelClassName());
            }
            else
            {
                UIManager.setLookAndFeel(UIManager
                        .getCrossPlatformLookAndFeelClassName());
            }
        }
        catch (Exception e)
        {
            LOG.error("", e);
        }
        UIDefaults defaults = UIManager.getDefaults();
        defaults.put("TabbedPane.contentBorderInsets", new Insets(0, 0, 1, 0));
    }

    public static void main(String[] args)
    {
        LOG.info("\tOS: " + System.getProperty("os.name"));
        LOG.info("\tJava vendor: " + System.getProperty("java.vendor"));
        LOG.info("\tJava version: " + System.getProperty("java.version"));
        LOG.info("\tExecution data model: "
                + System.getProperty("sun.arch.data.model") + "bit");

        initMacCompatibilityMode();
        setupLF();
        if (Xpcom.isMacOSX())
        {
            final String currentCP = System.getProperty("java.class.path");
            final String appleJavaExtentions = ":/System/Library/Java";
            System.setProperty("java.class.path", currentCP
                    + appleJavaExtentions);
        }
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);

        Xpcom.initialize();

        ToolTipManager.sharedInstance().setLightWeightPopupEnabled(false);
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                final JxBrowserDemo demo = new JxBrowserDemo();
                demo.createGUI();
            }
        });

    }

    private static void initMacCompatibilityMode() {
        if (PlatformContext.isMacOS()) {
            System.setProperty("com.apple.eawt.CocoaComponent.CompatibilityMode",
                    String.valueOf(PlatformContext.isTiger()));
        }
    }

    public JxBrowserDemo()
    {
        _childCreator = new ChildWindowCreator();
    }

    private WebBrowser createBrowser()
    {
        WebBrowser webBrowser = WebBrowserFactory.getInstance().createBrowser();
        navigateWebBrowser(webBrowser, _aboutBlank);

        configureNewBrowser(webBrowser);
        _browsersPane.addBrowser(webBrowser);
        return webBrowser;
    }

    private void navigateWebBrowser(final WebBrowser webBrowser,
            final String url)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                webBrowser.navigate(url);
            }
        });
    }

    private class MyAssyncHandler extends DefaultAsyncHandler
    {
        private final RuntimeException invocationPoint = new RuntimeException(
                "Xpcom asynchronous invocation failed");

        @Override
        public void after(Throwable t)
        {
            if (t != null)
            {
                invocationPoint.initCause(t);
                invocationPoint.printStackTrace();
            }
        }

    }

    private final MyAssyncHandler handler = new MyAssyncHandler();

    private void configureNewBrowser(final WebBrowser webBrowser)
    {
        webBrowser.addContentHandler(new DownloadContentHandler(webBrowser));
        webBrowser.setAttributes(OptionsDialog.DOC_ATTR);

        // Add context menu (pop menu on links, images so on.)
        _myContextMenuListener = new MyContextMenuListener(webBrowser);
        webBrowser.addContextMenuListener(_myContextMenuListener);

        webBrowser.setWindowCreator(_childCreator);

        AsyncHandlerFactory assyncHandlerFactory = new AsyncHandlerFactory()
        {

            public AsyncHandler createHandler()
            {
                return handler;
            }
        };

        webBrowser.setAsyncHandler(assyncHandlerFactory);

        webBrowser.addStatusChangeListener(new StatusChangeListener()
        {
            public void statusChanged(final StatusChangeEvent event)
            {

                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        _statusText.setText(event.getStatus());
                    }
                });
            }
        });

        webBrowser.addRequestListener(new BrowserEventCollector());
    }

    private void setButtonsState(final WebBrowser browser)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                _goBackAction.setEnabled(browser.getHistory().getCanGoBack());
                _goForwardAction.setEnabled(browser.getHistory()
                        .getCanGoForward());
            }
        });
    }

    private void setLocation(final String url)
    {
        _locationBox.setURLText(url);
        _locationBox.getMyEditor().setFocusAndSelectedText(STATE_TAB_CHANGED);
    }

    private void setStatusBarText()
    {
        _statusText.setText("");
    }

    private void setLoadImage()
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                _loaderImage.setLoadingMode(false);
            }

        });
    }

    private boolean isActiveWebBrowser(WebBrowser webBrowser)
    {
        return webBrowser.getComponent().equals(
                _browsersPane.getActiveBrowser().getComponent());
    }

    private void createGUI()
    {
        _closeCurrentBrowserAction = new CloseCurrentBrowserAction();
        _createNewBrowserTabAction = new CreateNewBrowserTabAction();
        _goBackAction = new GoBackAction();
        _goForwardAction = new GoForwardAction();
        _refreshAction = new RefreshAction();
        _stopAction = new StopAction();
        _homeAction = new GoHomeAction();
        _pageSetup = new PageSetupAction();
        _printPreview = new PrintPreviewAction();
        _print = new PrintAction();
        _loaderImage = new LoaderImage();
        _progressBar = new JProgressBar();
        _save = new SaveAction();
        _open = new OpenAction();
        _showDownloads = new ShowDownloadsAction();
        _cutAction = new CutAction();
        _copyAction = new CopyAction();
        _pasteAction = new PasteAction();
        _deleteAction = new DeleteAction();
        _saveAsImage = new SaveAsImageAction();

        JPopupMenu browserMenu = new JPopupMenu();
        browserMenu.add(_createNewBrowserTabAction);
        browserMenu.add(_closeCurrentBrowserAction);
        browserMenu.addPopupMenuListener(new PopupMenuListener()
        {
            public void popupMenuCanceled(PopupMenuEvent e)
            {
            }

            public void popupMenuWillBecomeInvisible(PopupMenuEvent e)
            {
            }

            public void popupMenuWillBecomeVisible(PopupMenuEvent e)
            {
                _closeCurrentBrowserAction.setEnabled(_browsersPane
                        .getTabCount() > 1);
            }
        });

        final JFrame frame = new JFrame();
        _sourceAction = new SourceAction(frame);
        JMenuItem newTab = new JMenuItem(_createNewBrowserTabAction);
        JMenuItem closeTab = new JMenuItem(_closeCurrentBrowserAction);
        JMenuItem pageSetup = new JMenuItem(_pageSetup);
        JMenuItem printPreview = new JMenuItem(_printPreview);
        JMenuItem print = new JMenuItem(_print);
        JMenuItem save = new JMenuItem(_save);
        JMenuItem open = new JMenuItem(_open);
        JMenuItem connectionSettingsMenuItem = new JMenuItem(
                new OptionsDialogAction(frame));
        JMenuItem exit = new JMenuItem(new ExitApplicationAction());
        JMenu fileMenu = new JMenu("File");
        fileMenu.add(newTab);
        fileMenu.addSeparator();
        fileMenu.add(open);
        fileMenu.add(save);
        if (!Xpcom.isMacOSX())
        {
            fileMenu.addSeparator();
            //fileMenu.add(pageSetup);
            fileMenu.add(printPreview);
            fileMenu.add(print);
        }
        fileMenu.addSeparator();
        // fileMenu.add(new FindAction());
        fileMenu.add(_saveAsImage);
        fileMenu.add(closeTab);
        fileMenu.add(connectionSettingsMenuItem);
        fileMenu.addSeparator();
        fileMenu.add(exit);

        JMenu editMenu = new JMenu("Edit");
        editMenu.add(_cutAction);
        editMenu.add(_copyAction);
        editMenu.add(_pasteAction);
        editMenu.add(_deleteAction);
        editMenu.addSeparator();
        editMenu.add(new SelectAllAction());
        editMenu.addMenuListener(new MenuListener()
        {
            public void menuSelected(MenuEvent event)
            {
                if (_locationBox.getMyEditor().getInputBox().hasFocus())
                {
                    _HasFocusInLocationCombo = true;
                    setEnabledEditMenuActionForLocationCombo();
                }
                else
                {
                    _HasFocusInLocationCombo = false;
                    setEnabledEditMenuActionForWebBrowserComponent();
                }
            }

            public void menuDeselected(MenuEvent e)
            {
                _browsersPane.getActiveBrowser().activate();
            }

            public void menuCanceled(MenuEvent e)
            {
                _browsersPane.getActiveBrowser().activate();
            }
        });

        JMenuItem _refreshMenuItem = new JMenuItem(_refreshAction);
        JMenuItem _stopMenuItem = new JMenuItem(_stopAction);
        JMenuItem sourceMenuItem = new JMenuItem(_sourceAction);
        JMenuItem dowloadsMenuItem = new JMenuItem(_showDownloads);
        JMenuItem pageDown = new JMenuItem(new PageDownAction());
        JMenuItem pageUP = new JMenuItem(new PageUpAction());
        JMenu view = new JMenu("View");
        view.add(_refreshMenuItem);
        view.add(_stopMenuItem);
        view.addSeparator();
        view.add(dowloadsMenuItem);
        view.addSeparator();
        view.add(sourceMenuItem);
        view.addSeparator();
        view.add(pageDown);
        view.add(pageUP);

        JMenuItem _backMenuItem = new JMenuItem(_goBackAction);
        JMenuItem _forwardMenuItem = new JMenuItem(_goForwardAction);
        JMenuItem goHome = new JMenuItem(new GoHomeAction());
        JMenu go = new JMenu("Go");
        go.add(_backMenuItem);
        go.add(_forwardMenuItem);
        go.add(goHome);

        final JMenuItem about = new JMenuItem(new ShowAboutAction(frame));
        JMenu help = new JMenu("Help");
        help.add(about);

        JMenuBar mainMenu = new JMenuBar();
        mainMenu.add(fileMenu);
        mainMenu.add(editMenu);
        mainMenu.add(view);
        mainMenu.add(go);
        mainMenu.add(help);

        if (!Xpcom.isMacOSX())
        {
            JPanel menuPanel = new JPanel(new BorderLayout());
            menuPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
            _loaderImage.setLoadingMode(false);
            menuPanel.add(_loaderImage);
            menuPanel.setOpaque(false);
            mainMenu.add(menuPanel);
        }

        JPanel browsersPanel = new JPanel(new BorderLayout());
        printPreviewPanel = new JPanel(new BorderLayout());

        printPreviewLayout = new CardLayout();
        contentPanel = new JPanel(printPreviewLayout);
        contentPanel.add("browsersPanel",browsersPanel);
        contentPanel.add("printPreviewPanel", printPreviewPanel);

        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(contentPanel, BorderLayout.CENTER);

        JPanel toolPanel = new JPanel();
        GroupLayout layout = new GroupLayout(toolPanel);
        toolPanel.setLayout(layout);
        browsersPanel.add(toolPanel, BorderLayout.NORTH);

        JButton backButton = createButton("Back", _goBackAction);
        backButton.setEnabled(false);
        JButton forwardButton = createButton("Forward", _goForwardAction);
        forwardButton.setEnabled(false);
        JButton refreshButton = createButton("Refresh", _refreshAction);
        JButton stopButton = createButton("Stop", _stopAction);
        JButton homeButton = createButton("Home", _homeAction);
        JSeparator horizontalSeparator = new JSeparator(JSeparator.HORIZONTAL);
        _locationBox = new LocationCombo();
        _locationBox.getMyEditor().getInputBox().setFont(
                new Font(_locationBox.getMyEditor().getInputBox().getFont()
                        .getFamily(), mainMenu.getFont().getStyle(),
                        _locationBox.getMyEditor().getInputBox().getFont()
                                .getSize()));

        GroupLayout.ParallelGroup paralelGroupForHorizontal = layout
                .createParallelGroup();
        GroupLayout.SequentialGroup sequentialGroupForVertical = layout
                .createSequentialGroup();

        // gorizontal layout
        GroupLayout.SequentialGroup sequentialGroupForHorizontal = layout
                .createSequentialGroup();
        sequentialGroupForHorizontal.add(3);
        sequentialGroupForHorizontal.add(backButton);
        sequentialGroupForHorizontal.add(3);
        sequentialGroupForHorizontal.add(forwardButton);
        sequentialGroupForHorizontal.add(9);
        sequentialGroupForHorizontal.add(refreshButton);
        sequentialGroupForHorizontal.add(3);
        sequentialGroupForHorizontal.add(stopButton);
        sequentialGroupForHorizontal.add(9);
        sequentialGroupForHorizontal.add(homeButton);
        sequentialGroupForHorizontal.add(7);
        sequentialGroupForHorizontal.add(_locationBox, 10, 10, 10000);
        sequentialGroupForHorizontal.add(3);

        paralelGroupForHorizontal.add(sequentialGroupForHorizontal);
        paralelGroupForHorizontal.add(horizontalSeparator);

        // vertical layout
        GroupLayout.ParallelGroup parallelGroupForVertical;
        if (Xpcom.isMacOSX())
        {
            parallelGroupForVertical = layout
                    .createParallelGroup(GroupLayout.TRAILING);
        }
        else
        {
            parallelGroupForVertical = layout
                    .createParallelGroup(GroupLayout.BASELINE);
        }
        parallelGroupForVertical.add(backButton);
        parallelGroupForVertical.add(forwardButton);
        parallelGroupForVertical.add(refreshButton);
        parallelGroupForVertical.add(stopButton);
        parallelGroupForVertical.add(homeButton);
        parallelGroupForVertical.add(_locationBox);

        if (Xpcom.isMacOSX())
        {
            sequentialGroupForVertical.add(11, 11, 11);
        }
        else
        {
            sequentialGroupForVertical.add(3, 3, 3);
        }
        sequentialGroupForVertical.add(parallelGroupForVertical);
        sequentialGroupForVertical.add(3, 3, 3);
        sequentialGroupForVertical.add(horizontalSeparator, 2, 2, 2);

        layout.setHorizontalGroup(paralelGroupForHorizontal);
        layout.setVerticalGroup(sequentialGroupForVertical);

        final AbstractAction[] actions = { _goBackAction, _goForwardAction,
                _stopAction };
        _browsersPane = new TabbedBrowsersPane(_progressBar, browserMenu,
                _closeCurrentBrowserAction, actions);

        _browsersPane.addChangeListener(new ChangeListener()
        {
            public void stateChanged(ChangeEvent e)
            {
                final WebBrowser activeBrowser = _browsersPane
                        .getActiveBrowser();
                if (null == activeBrowser)
                {
                    return;
                }

                setButtonsState(activeBrowser);
                setStatusBarText();
                setLoadImage();
                activeBrowser.setAttributes(OptionsDialog.DOC_ATTR);
                setLocation(activeBrowser.getLocationURL());
            }
        });

        _locationBox.addLocationListener(new LocationListener()
        {
            public void browserNavigate(final String newLocation)
            {
                final WebBrowser activeBrowser = _browsersPane
                        .getActiveBrowser();

                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        AsyncHandlerFactory oldAsyncHandler = activeBrowser
                                .getAsyncHandler();
                        try
                        {
                            activeBrowser
                                    .setAsyncHandler(new DefaultAsyncHandler()
                                    {
                                        @Override
                                        public void after(Throwable t)
                                        {
                                            if (t instanceof XPCOMException)
                                            {
                                                XPCOMException xpcomException = (XPCOMException) t;
                                                LOG
                                                        .warn(String
                                                                .format(
                                                                        "Navigation failed with error code: 0x%x",
                                                                        xpcomException.errorcode));
                                            }
                                            else
                                            {
                                                super.after(t);
                                            }
                                        }
                                    });
                            activeBrowser.navigate(newLocation);
                        }
                        finally
                        {
                            activeBrowser.setAsyncHandler(oldAsyncHandler);
                        }
                    }
                });
            }
        });

        _statusText.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 8,
                0, 0));
        _statusPanel.setLayout(new BorderLayout());
        _statusPanel.add(_statusText, BorderLayout.WEST);
        _statusPanel.add(_progressBar, BorderLayout.EAST);
        _statusPanel.setPreferredSize(new Dimension(20, 18));
        LineBevel horisontalBevel = new LineBevel();
        _statusPanel.add(horisontalBevel, BorderLayout.NORTH);

        browsersPanel.add(_statusPanel, BorderLayout.SOUTH);
        browsersPanel.add(_browsersPane, BorderLayout.CENTER);

        createBrowser();

        frame.setJMenuBar(mainMenu);
        frame.setTitle("JxBrowser Demo");
        frame.setSize(830, 600);
        frame.setLocationRelativeTo(null);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        final ImageIcon icon = new ImageIcon(getClass().getResource(
                "res/jxbrowser16x16.png"));
        frame.setIconImage(icon.getImage());

        frame.addWindowListener(new WindowAdapter()
        {
            @Override
            public void windowOpened(WindowEvent e)
            {
                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        _homeAction.actionPerformed(null);
                    }
                });
            }

            @Override
            public void windowClosing(WindowEvent event)
            {
                JFrame frame = (JFrame) event.getSource();
                frame.setVisible(false);
                try
                {
                    _browsersPane.disposeBrowsers();
                }
                catch (InterruptedException e)
                {
                    LOG.error("Can`t stop browsers", e);
                    throw new RuntimeException(e);
                }
                frame.dispose();
                System.exit(0);
            }
        });
        frame.setVisible(true);
    }

    private void setEnabledEditMenuActionForLocationCombo()
    {
        String selectedText = _locationBox.getMyEditor().getInputBox()
                .getSelectedText();
        if (selectedText != null && !selectedText.equals(""))
        {
            _cutAction.setEnabled(true);
            _copyAction.setEnabled(true);
            _deleteAction.setEnabled(true);
        }
        else
        {
            _cutAction.setEnabled(false);
            _copyAction.setEnabled(false);
            _deleteAction.setEnabled(false);
        }

        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        DataFlavor flavor = DataFlavor.stringFlavor;
        if (clipboard.isDataFlavorAvailable(flavor))
        {
            _pasteAction.setEnabled(true);
        }
        else
        {
            _pasteAction.setEnabled(false);
        }
    }

    private void setEnabledEditMenuActionForWebBrowserComponent()
    {
        WebBrowser webBrowser = _browsersPane.getActiveBrowser();
        _cutAction.setEnabled(webBrowser.getClipboardSupport()
                .canCutSelection());
        _copyAction.setEnabled(webBrowser.getClipboardSupport()
                .canCopySelection());
        _pasteAction.setEnabled(webBrowser.getClipboardSupport()
                .canPasteClipboardContent());
        _deleteAction.setEnabled(webBrowser.getClipboardSupport()
                .canCutSelection());

        _browsersPane.getActiveBrowser().deactivate();
        if (webBrowser.getClipboardSupport().canCutSelection())
        {
            _browsersPane.getActiveBrowser().getClipboardSupport()
                    .setDisplaySelection();
        }
    }

    private JButton createButton(String caption, Action action)
    {
        ActionButton button = new ActionButton(caption, action);
        button.setPreferredSize(new Dimension(23, 20));
        final String imageName = caption.toLowerCase();
        button.setIcon(new ImageIcon(getClass().getResource(
                "res/" + imageName + ".png")));
        button.setRolloverIcon(new ImageIcon(getClass().getResource(
                "res/" + imageName + "A.png")));
        button.setName("button");

        return button;
    }

    @SuppressWarnings("serial")
    private class CloseCurrentBrowserAction extends AbstractAction
    {
        public CloseCurrentBrowserAction()
        {
            super("Close Tab");
            setEnabled(false);
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_W,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent e)
        {
            _browsersPane.removeBrowser();
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    _closeCurrentBrowserAction.setEnabled(_browsersPane
                            .getTabCount() > 1);
                }
            });
        }
    }

    @SuppressWarnings("serial")
    private class CreateNewBrowserTabAction extends AbstractAction
    {
        CreateNewBrowserTabAction()
        {
            super("New Tab");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_T,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent e)
        {
            _closeCurrentBrowserAction.setEnabled(true);
            createBrowser();
        }
    }

    @SuppressWarnings("serial")
    private class ExitApplicationAction extends AbstractAction
    {
        ExitApplicationAction()
        {
            super("Exit");
        }

        public void actionPerformed(ActionEvent e)
        {
            System.exit(0);
        }
    }

    private class GoHomeAction extends AbstractAction
    {
        private static final String HOME_URL = "http://www.teamdev.com";

        GoHomeAction()
        {
            super("Home");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_HOME,
                    InputEvent.ALT_MASK));
        }

        public void actionPerformed(ActionEvent e)
        {
            WebBrowser browser = _browsersPane.getActiveBrowser();
            browser.navigate(HOME_URL);
        }
    }

    public class SaveAsImageAction extends AbstractAction
    {
        private String extensionPNG = ".png";
        private SaveJFileChooser fileChooser;

        public SaveAsImageAction()
        {
            super("Render to PNG...");
        }

        public void actionPerformed(ActionEvent e)
        {
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    fileChooser = new SaveJFileChooser(_browsersPane, action,
                            false);
                    fileChooser
                            .setSelectedFile(new File("image" + extensionPNG));
                    fileChooser.addChoosableFileFilter(new FileFilter()
                    {
                        public String getDescription()
                        {
                            return "PNG (*" + extensionPNG + ")";
                        }

                        public boolean accept(File file)
                        {
                            return file.isDirectory()
                                    || file.getPath().lastIndexOf(extensionPNG) != -1;
                        }
                    });
                    fileChooser.setVisible(true);
                }
            });
        }

        private Runnable action = new Runnable()
        {
            public void run()
            {
                File selectedFile = fileChooser.getSelectedFile();
                File filePng;
                if (-1 == selectedFile.getName().lastIndexOf(extensionPNG))
                {
                    filePng = new File(selectedFile.getAbsolutePath()
                            + extensionPNG);
                }
                else
                {
                    filePng = selectedFile;
                }
                BufferedImage image = _browsersPane.getActiveBrowser()
                        .toImage();
                try
                {
                    ImageIO.write(image, "PNG", filePng);
                }
                catch (IOException exception)
                {
                    JOptionPane
                            .showMessageDialog(
                                    _browsersPane,
                                    "<html><b>Cannot create image for this web page</b><p>"
                                            + "Make sure the web page is loaded completely and try again.");
                }
            }
        };
    }

    private class RefreshAction extends AbstractAction
    {
        RefreshAction()
        {
            super("Refresh");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
        }

        public void actionPerformed(ActionEvent e)
        {
            _browsersPane.getActiveBrowser().refresh();
        }
    }

    private class StopAction extends AbstractAction
    {
        StopAction()
        {
            super("Stop");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(
                    KeyEvent.VK_ESCAPE, 0));
        }

        public void actionPerformed(ActionEvent e)
        {
            _browsersPane.getActiveBrowser().stop();
        }
    }

    private class GoBackAction extends AbstractAction
    {
        GoBackAction()
        {
            super("Back");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,
                    InputEvent.ALT_MASK));
        }

        public void actionPerformed(ActionEvent e)
        {
            _browsersPane.getActiveBrowser().goBack();
        }
    }

    private class GoForwardAction extends AbstractAction
    {
        GoForwardAction()
        {
            super("Forward");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT,
                    InputEvent.ALT_MASK));
        }

        public void actionPerformed(ActionEvent e)
        {
            _browsersPane.getActiveBrowser().goForward();
        }
    }

    private class FindAction extends AbstractAction
    {

        FindAction()
        {
            super("Find");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent e)
        {
            /*
             * String searchString = (String) JOptionPane.showInputDialog(
             * _browsersPane.getParent(), "Find phrase", "Find",
             * JOptionPane.QUESTION_MESSAGE, new ImageIcon(), null, null);
             * boolean result = _browsersPane.getActiveBrowser().findString(
             * searchString); bif (!result) {
             * JOptionPane.showMessageDialog(_browsersPane
             * .getTopLevelAncestor(), "Phrase not found!"); }
             */
        }
    }

    private class SourceAction extends AbstractAction
    {
        private final JFrame _frame;

        public SourceAction(JFrame frame)
        {
            super("Page Source");
            _frame = frame;
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_U,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent e)
        {
            WebBrowser webBrowser = _browsersPane.getActiveBrowser();

            if (webBrowser != null)
            {
                SourceFrame sourceFrame = new SourceFrame(webBrowser,
                        (int) (_frame.getLocation().getX() + 15), (int) (_frame
                                .getLocation().getY() + 110));
                sourceFrame
                        .setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
            }

        }
    }

    private class ShowAboutAction extends AbstractAction
    {
        Window _owner;

        ShowAboutAction(Window owner)
        {
            super("About JxBrowser...");
            _owner = owner;
        }

        String[] manifestInfo()
        {

            String result[] = new String[2];
            String classpath = System.getProperty("java.class.path");
            String path[] = classpath.split(PATH_SEPARATOR);
            String jarFileName = null;
            for (String jarfile : path)
            {
                if (jarfile.endsWith("demo.jar"))
                {
                    jarFileName = jarfile;
                    break;
                }
            }
            if (null == jarFileName)
            {
                return null;
            }

            File file = new File(jarFileName);
            try
            {
                JarFile jarFile = new JarFile(file);
                result[0] = jarFile.getManifest().getMainAttributes().getValue(
                        "Version");
                result[1] = jarFile.getManifest().getMainAttributes().getValue(
                        "Build");
                return result;
            }
            catch (IOException e)
            {
                LOG.error(e);
            }

            return null;
        }

        public void actionPerformed(ActionEvent e)
        {
            final String java = System.getProperty("java.version");
            final String osname = System.getProperty("os.name");

            String version[] = manifestInfo();
            if (null == version)
            {
                version = new String[] { "1.3", "developer.local.build" };
            }
            AboutDialog aboutDialog = new AboutDialog(
                    (Frame) _owner,
                    "About JxBrowser Demo",
                    "<html><H2>JxBrowser Demo</H2>"
                            + "<p>Version: "
                            + version[0]
                            + "</p><p>Build number: "
                            + version[1]
                            + "<br></p><p>\u00a9 2005-2009 TeamDev Ltd.</p><br>System: "
                            + osname
                            + "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Java "
                            + java + "</html>", new ImageIcon(getClass()
                            .getResource("res/Logo.png")));
            aboutDialog.setVisible(true);
        }
    }

    private class OptionsDialogAction extends AbstractAction
    {
        Frame _owner;

        OptionsDialogAction(Frame owner)
        {
            super("Options...");
            _owner = owner;
        }

        public void actionPerformed(ActionEvent e)
        {
            OptionsDialog dialog = new OptionsDialog(_owner);
            dialog.pack();
            dialog.setLocationRelativeTo(_owner);
            dialog.setVisible(true);
            _browsersPane.getActiveBrowser().setAttributes(
                    OptionsDialog.DOC_ATTR);
        }
    }

    private class ChildWindowCreator implements WindowCreator
    {

        public WebBrowserWindow createChildWindow(Component parentComponent,
                long flags)
        {
            if ((flags & nsIWebBrowserChrome.CHROME_OPENAS_CHROME) == 0)
            {
                return new TabWebBrowserWindow();
            }
            final WindowCreator defaultWindowCreator = WebBrowserFactory
                    .getInstance().getDefaultWindowCreator();
            return defaultWindowCreator.createChildWindow(parentComponent,
                    flags);
        }

        private class TabWebBrowserWindow implements WebBrowserWindow
        {
            private WebBrowser _webBrowser;

            boolean _visible;

            boolean _closed = false;

            public void setVisible(boolean visible)
            {
                _webBrowser.getComponent().setVisible(visible);
                _visible = visible;
            }

            public void setSize(int width, int height)
            {
                _webBrowser.getComponent().setSize(width, height);
            }

            public void setWebBrowser(WebBrowser webBrowser)
            {
                _webBrowser = webBrowser;
                configureNewBrowser(webBrowser);
                _browsersPane.addBrowser(webBrowser);
            }

            public void setModal(boolean modal)
            {
            }

            public void close()
            {
                _browsersPane.removeBrowser(_webBrowser);
                _closed = true;
            }

            public boolean isVisible()
            {
                return _visible;
            }

            public boolean isClosed()
            {
                return _closed;
            }

            public boolean isResizable()
            {
                throw new RuntimeException(new OperationNotSupportedException(
                        "isResizable not supported by this implementation"));
            }

            public void setResizable(boolean resizable)
            {
                throw new RuntimeException(new OperationNotSupportedException(
                        "setResizable not supported by this implementation"));
            }

        }
    }

    private class SaveAction extends AbstractAction
    {
        SaveAction()
        {
            super("Save As...");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_S,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent e)
        {
            final WebBrowser browser = _browsersPane.getActiveBrowser();
            browser.showSaveFileDialog();
        }
    }

    private class OpenAction extends AbstractAction
    {
        OpenAction()
        {
            super("Open File...");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_O,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent e)
        {
            final WebBrowser browser = _browsersPane.getActiveBrowser();
            browser.showOpenFileDialog();
        }
    }

    private class PageSetupAction extends AbstractAction
    {
        PageSetupAction()
        {
            super("Page Setup...");
        }

        public void actionPerformed(ActionEvent e)
        {
            WebBrowserPrinting printingAPI = _browsersPane.getActiveBrowser()
                    .getPrinting();
            printingAPI.showPageSetupDialog();
        }
    }

    private class PrintPreviewAction extends AbstractAction
    {
        PrintPreviewAction()
        {
            super("Print Preview");
        }

        public void actionPerformed(ActionEvent e)
        {
            final WebBrowser browser = _browsersPane.getActiveBrowser();
            browser.stop();
            JFrame frame = (JFrame) _browsersPane.getTopLevelAncestor();
            printPreviewPanel.add(new PrintPreviewPanel(
                                        frame,
                                        browser,
                                        new Runnable() {
                                            public void run()
                                            {
                                                printPreviewPanel.removeAll();
                                                printPreviewLayout.next(contentPanel);
                                            }
                                        }
            ), BorderLayout.CENTER);
            printPreviewLayout.next(contentPanel);
        }

    }

    private class PrintAction extends AbstractAction
    {
        PrintAction()
        {
            super("Print...");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_P,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent e)
        {
            new PrintDialog((JFrame) _browsersPane.getTopLevelAncestor(),
                    _browsersPane.getActiveBrowser());
        }
    }

    private class ShowDownloadsAction extends AbstractAction
    {
        ShowDownloadsAction()
        {
            super("Downloads");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_J,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent e)
        {
            DownloadManager manager = DownloadManagerFactory.getInstance()
                    .getDownloadManager(_browsersPane.getActiveBrowser());
            manager.showDownloadManagerWindow();
        }
    }

    private class MyContextMenuListener implements ContextMenuListener
    {
        private final ContextMenuProvider _contextMenuProvider;

        public MyContextMenuListener(WebBrowser webBrowser)
        {
            Action[] actions = { _goBackAction, _goForwardAction,
                    _refreshAction, _stopAction };
            _contextMenuProvider = new ContextMenuProvider(webBrowser, actions);
            _contextMenuProvider
                    .addOpenLinkInNewTabAction(new OpenLinkInNewTab());
            _contextMenuProvider
                    .addLinkPropertiesAction(new PropertiesAction());
            _contextMenuProvider
                    .addImagePropertiesAction(new PropertiesAction());
            _contextMenuProvider
                    .addLinkImagePropertiesAction(new PropertiesAction());
            _contextMenuProvider.addSaveImageAs(new SaveImageAction());
            _contextMenuProvider.addSourceAction(_sourceAction);
        }

        public void contextMenuRequested(ContextMenuEvent event)
        {
            _contextMenuProvider.setContextMenuEvent(event);
            _contextMenuProvider.createMenu();
            Component browserComponent = event.getWebBrowser().getComponent();
            if (browserComponent.isShowing())
            {
                final int x = event.getX();
                final int y = event.getY();
                _contextMenuProvider.show(browserComponent, x, y);
            }
            else
            {
                LOG
                        .warn("Context menu requested for a browser that is not showing.");
            }
        }

        private class PropertiesAction extends AbstractAction
        {
            public void actionPerformed(ActionEvent e)
            {
                PropertiesDialog propertiesDialog = new PropertiesDialog(
                        _contextMenuProvider.getContextMenuEvent());
                propertiesDialog
                        .setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
                propertiesDialog.setVisible(true);
            }
        }

        private class OpenLinkInNewTab extends AbstractAction
        {
            public OpenLinkInNewTab()
            {
                super("Open Link in New Tab");
            }

            public void actionPerformed(ActionEvent e)
            {
                final WebBrowser browser = createBrowser();
                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        browser.navigate(_contextMenuProvider
                                .getContextMenuEvent().getAssociatedLink());
                    }
                });
            }
        }

        private class SaveImageAction extends AbstractAction
        {
            File getFile(URL from)
            {
                JFileChooser saveDialog = new JFileChooser();
                saveDialog.setDialogTitle("Save Image As");
                File selected = new File(from.getPath());
                saveDialog.setSelectedFile(selected);
                int ret = saveDialog.showSaveDialog(_contextMenuProvider
                        .getContextMenuEvent().getWebBrowser().getComponent());
                if (ret != JFileChooser.APPROVE_OPTION)
                {
                    return null;
                }
                else
                {
                    return saveDialog.getSelectedFile();
                }
            }

            private class SaveEventListener implements RequestListener
            {
                JFrame frame = (JFrame) _browsersPane.getTopLevelAncestor();
                ProgressDialog progressDialog = new ProgressDialog(frame, "Save image progress");

                void updateDialog(final int progress)
                {
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            progressDialog.getProgressBar().setValue(progress);
                            progressDialog.setVisible(true);
                        }
                    });
                }

                public void locationChanged(LocationEvent event)
                {
                }

                public void progressChanged(ProgressEvent event)
                {
                    synchronized (event)
                    {
                        int maxTotalProgress = (int) event
                                .getMaxTotalProgress();
                        progressDialog.getProgressBar().setMaximum(
                                maxTotalProgress);
                    }
                    int curProgress = (int) event.getCurTotalProgress();
                    updateDialog(curProgress);
                }

                public void securityChanged(SecurityEvent event)
                {
                }

                public void stateChanged(StateEvent event)
                {
                    if (event.getStatus() == 0)
                    {
                        SwingUtilities.invokeLater(new Runnable()
                        {
                            public void run()
                            {
                                progressDialog.dispose();
                            }
                        });
                    }
                }

                public void statusChanged(StatusEvent event)
                {
                }
            }

            public void actionPerformed(ActionEvent e)
            {
                final HTMLImageElement imageElement = (HTMLImageElement) _contextMenuProvider
                        .getContextMenuEvent().getTargetNode();
                try
                {
                    URL url = new URL(imageElement.getSrc());
                    WebBrowser browser = _contextMenuProvider
                            .getContextMenuEvent().getWebBrowser();
                    File file = getFile(url);
                    if (file != null)
                    {
                        browser.saveURLtoFile(url, file,
                                new SaveEventListener());
                    }
                }
                catch (Exception e1)
                {
                    e1.printStackTrace();
                }
            }
        }

    }

    private class CutAction extends AbstractAction
    {
        public CutAction()
        {
            super("Cut");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_X,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent event)
        {
            if (_HasFocusInLocationCombo)
            {
                new DefaultEditorKit.CutAction().actionPerformed(event);
            }
            else
            {
                _browsersPane.getActiveBrowser().getClipboardSupport()
                        .cutSelection();
            }
        }
    }

    private class CopyAction extends AbstractAction
    {
        public CopyAction()
        {
            super("Copy");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_C,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent event)
        {
            if (_HasFocusInLocationCombo)
            {
                new DefaultEditorKit.CopyAction().actionPerformed(event);
            }
            else
            {
                _browsersPane.getActiveBrowser().getClipboardSupport()
                        .copySelectionToClipboard();
            }
        }
    }

    private class PasteAction extends AbstractAction
    {
        public PasteAction()
        {
            super("Paste");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_V,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent event)
        {
            // if system shotcup is enabled
            if (event.getModifiers() != MENU_ACSELERATOR_KEY)
            {
                if (_HasFocusInLocationCombo)
                {
                    new DefaultEditorKit.PasteAction().actionPerformed(event);
                }
                else
                {
                    _browsersPane.getActiveBrowser().getClipboardSupport()
                            .pasteClipboardContent();
                }
            }
        }
    }

    private class DeleteAction extends AbstractAction
    {
        public DeleteAction()
        {
            super("Delete");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(
                    KeyEvent.VK_DELETE, 0));
        }

        public void actionPerformed(ActionEvent event)
        {
            if (_HasFocusInLocationCombo)
            {
                String allText = _locationBox.getMyEditor().getInputBox()
                        .getText();
                String selectedText = _locationBox.getMyEditor().getInputBox()
                        .getSelectedText();
                if (selectedText != null)
                {
                    final String text = allText.substring(0, allText
                            .indexOf(selectedText))
                            + allText.substring(allText.indexOf(selectedText)
                                    + selectedText.length(), allText.length());
                    SwingUtilities.invokeLater(new Runnable()
                    {

                        public void run()
                        {
                            _locationBox.getMyEditor().getInputBox().setText(
                                    text);
                        }

                    });
                }
            }
            else
            {
                _browsersPane.getActiveBrowser().getClipboardSupport()
                        .deleteSelection();
            }
        }
    }

    private class SelectAllAction extends AbstractAction
    {
        public SelectAllAction()
        {
            super("Select All");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_A,
                    MENU_ACSELERATOR_KEY));
        }

        public void actionPerformed(ActionEvent event)
        {
            if (_HasFocusInLocationCombo)
            {
                _locationBox.getMyEditor().getInputBox().selectAll();
            }
            else
            {
                _browsersPane.getActiveBrowser().getClipboardSupport()
                        .selectAll();
            }
        }
    }

    private class PageDownAction extends AbstractAction
    {

        public PageDownAction()
        {
            super("Page Down");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(
                    KeyEvent.VK_PAGE_DOWN, InputEvent.ALT_DOWN_MASK));
        }

        public void actionPerformed(ActionEvent event)
        {
            _browsersPane.getActiveBrowser().pageDown();
        }
    }

    private class PageUpAction extends AbstractAction
    {

        public PageUpAction()
        {
            super("Page Up");
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(
                    KeyEvent.VK_PAGE_UP, InputEvent.ALT_DOWN_MASK));
        }

        public void actionPerformed(ActionEvent event)
        {
            _browsersPane.getActiveBrowser().pageUp();
        }
    }

    private class BrowserEventCollector extends RequestAdapter
    {
        public BrowserEventCollector()
        {
        }

        @Override
        public void locationChanged(final LocationEvent event)
        {
            if (isActiveWebBrowser(event.getWebBrowser()))
            {
                if (event.isLoadingDocument()
                        && !event.getLocation().equals(_aboutBlank))
                {
                    SwingUtilities.invokeLater(new Runnable()
                    {

                        public void run()
                        {
                            _locationBox.setURLText(event.getLocation());
                        }

                    });
                }
            }
        }

        @Override
        public void stateChanged(StateEvent event)
        {
            if (event.isLoadingCompleted())
            {
                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        _statusText.setText("Done");
                    }
                });
            }

            final boolean isLoading = event.isLoadingDocument();
            if (!isLoading && event.isStoped() && event.isWindow()
                    && event.isNetwork())
            {
                if (event.getWebBrowser().getLocationURL().equals(_aboutBlank)
                        && isActiveWebBrowser(event.getWebBrowser()))
                {
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            _browsersPane
                                    .setTitle(TabbedBrowsersPane._untitled);
                            _locationBox.getMyEditor().setFocusAndSelectedText(
                                    STATE_END_LOADING);
                        }
                    });
                }
            }

            final WebBrowser webBrowser = event.getWebBrowser();
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    if (isActiveWebBrowser(webBrowser))
                    {
                        _loaderImage.setLoadingMode(isLoading);
                    }
                }
            });
        }

        @Override
        public void statusChanged(final StatusEvent event)
        {
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    if (isActiveWebBrowser(event.getWebBrowser()))
                    {
                        _statusText.setText(event.getMessage());
                    }
                }
            });
        }
    }

}
