package com.teamdev.xpcom.mozilla.demo.components;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventObject;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.BevelBorder;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.teamdev.jxbrowser.printing.PaperSize;
import com.teamdev.jxbrowser.printing.PrintSettings.PageOrientation;

/**
 * The tool bar with widgets that control the print preview process
 */
class PrintPreviewToolBar extends JPanel
{

    public static interface ToolBarActionHandler<T extends EventObject>
    {
        public void performed(T event);
    }

    private static final long serialVersionUID = -960576875823621036L;
    /**
     * Handler for print button
     */
    private ToolBarActionHandler<ActionEvent> printBtnHandler;
    /**
     * Handler for page scroll spinner
     */
    private ToolBarActionHandler<ChangeEvent> scrollPageHandler;

    public class OrientationAction extends ActionEvent
    {

        private final PageOrientation orientation;

        public OrientationAction(Object source, int id, String command,
                PageOrientation orientation)
        {
            super(source, id, command);
            this.orientation = orientation;
        }

        public PageOrientation getOrientation()
        {
            return orientation;
        }

        private static final long serialVersionUID = -5054016618201294279L;

    }

    /**
     * Handler for page orientation radio buttons
     */
    private ToolBarActionHandler<OrientationAction> orientationChangedHandler;

    /**
     * Handler for paper size ComboBox
     */
    private ToolBarActionHandler<ActionEvent> paperSizeChangedHandler;

    /**
     * Handler for backgrounds ChekBox
     */
    private ToolBarActionHandler<ActionEvent> backgroudsFlagChangedHandler;

    /**
     * Handler for close button
     */
    private ToolBarActionHandler<ActionEvent> closeBtnHandler;

    private JPanel controlPanel;
    private JButton printBtn;
    private JSpinner scrollerSpinner;
    private SpinnerNumberModel pageScroller;
    private JLabel pageCount;
    private JRadioButton portraitRBtn;
    private JRadioButton landscapeRBtn;
    private JComboBox paperSizeCb;
    private JCheckBox backgroundsCb;
    private JButton closeBtn;

    /**
     * Action for print button
     * 
     */
    private class PrintAdaptor extends AbstractAction
    {
        private static final long serialVersionUID = -3346435676939151726L;

        PrintAdaptor()
        {
            super("Print...");
        }

        public void actionPerformed(ActionEvent e)
        {
            if (null != printBtnHandler) printBtnHandler.performed(e);
        }
    }

    /**
     * Change listener for page scroll pages spinner
     */
    private class ScrollPageAdaptor implements ChangeListener
    {
        public void stateChanged(ChangeEvent e)
        {
            if (null != scrollPageHandler)
            {
                scrollPageHandler.performed(e);
            }
        }
    }

    /**
     * Action listener for page orientation radio buttons
     */
    private class ChangeOrientationAdaptor implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            if (null != orientationChangedHandler) orientationChangedHandler
                    .performed(new OrientationAction(
                            e.getSource(),
                            e.getID(),
                            e.getActionCommand(),
                            e.getSource() == portraitRBtn ? PageOrientation.Portrait
                                    : PageOrientation.Landscape));
        }
    }

    /**
     * Action listener for paper size ComboBox
     */
    private class ChangePageSizeAdaptor implements ActionListener
    {

        public void actionPerformed(ActionEvent e)
        {
            if (null != paperSizeChangedHandler) paperSizeChangedHandler
                    .performed(e);
        }

    }

    /**
     * Action listener for backgrounds CheckBox
     */
    private class BackgroundsAdaptor extends AbstractAction
    {
        BackgroundsAdaptor()
        {
            super("Print backgrounds");
            putValue(TOOL_TIP_TEXT_KEY, "Print background colors and images");
        }

        public void actionPerformed(ActionEvent e)
        {
            if (null != backgroudsFlagChangedHandler) backgroudsFlagChangedHandler
                    .performed(e);
        }
        private static final long serialVersionUID = -8500945920267768938L;
    }

    /**
     * Action for close button
     */
    private class CloseAdaptor extends AbstractAction
    {

        private static final long serialVersionUID = -24289919366646003L;

        CloseAdaptor()
        {
            super("Close");
        }

        public void actionPerformed(ActionEvent e)
        {
            if (null != closeBtnHandler) closeBtnHandler.performed(e);
        }

    }

    /**
     * Construct the ControlToolBar
     */
    PrintPreviewToolBar()
    {
        super();
        setBorder(new CompoundBorder(new BevelBorder(BevelBorder.RAISED),
                new EmptyBorder(0, 3, 0, 3)));
        // The preferred size for print and close buttons
        Dimension btnPreferredSize = new Dimension(100, 24);

        // Panel for controlling the print preview
        controlPanel = new JPanel();
        printBtn = new JButton(new PrintAdaptor());
        printBtn.setPreferredSize(btnPreferredSize);
        this.pageScroller = new SpinnerNumberModel(1, 1, 1, 1)
        {
            @Override
            public Object getNextValue() {
                return super.getPreviousValue();
            }
            @Override
            public Object getPreviousValue() {
                return super.getNextValue();
            }
        };
        pageScroller.addChangeListener(new ScrollPageAdaptor());
        scrollerSpinner = new JSpinner(pageScroller);
        scrollerSpinner.setPreferredSize(new Dimension(50, 24));
        pageCount = new JLabel("of 1");

        portraitRBtn = new JRadioButton();
        landscapeRBtn = new JRadioButton();
        portraitRBtn.setIcon(new ImageIcon(getClass().getResource(
                "res/portrait.png")));
        portraitRBtn.setSelectedIcon(new ImageIcon(getClass().getResource(
                "res/portrait_sel.png")));
        portraitRBtn.setSelected(true);
        portraitRBtn.setToolTipText("Portrait");

        landscapeRBtn.setIcon(new ImageIcon(getClass().getResource(
                "res/landscape.png")));
                
      
        landscapeRBtn.setSelectedIcon(new ImageIcon(getClass().getResource(
                "res/landscape_sel.png")));
        landscapeRBtn.setToolTipText("Landscape");

        ChangeOrientationAdaptor orientationAdatptor = new ChangeOrientationAdaptor();
        portraitRBtn.addActionListener(orientationAdatptor);
        landscapeRBtn.addActionListener(orientationAdatptor);
        ButtonGroup group = new ButtonGroup();
        group.add(portraitRBtn);
        group.add(landscapeRBtn);

        final Vector<PaperSize> paperNames = new Vector<PaperSize>();
        paperNames.add(PaperSize.A4);
        paperNames.add(PaperSize.A3);
        paperNames.add(PaperSize.A5);
        paperSizeCb = new JComboBox(new DefaultComboBoxModel(paperNames));
        paperSizeCb.addActionListener(new ChangePageSizeAdaptor());

        closeBtn = new JButton(new CloseAdaptor());
        closeBtn.setPreferredSize(btnPreferredSize);

        backgroundsCb = new JCheckBox(new BackgroundsAdaptor());

        class MySeparator extends JSeparator
        {

            public MySeparator()
            {
                super(VERTICAL);
                setPreferredSize(new Dimension(2,25));
            }
            
        }
        
        // layout components
        setLayout(new BorderLayout());
        controlPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
        controlPanel.add(printBtn);
        controlPanel.add(new JLabel("Pages"));
        controlPanel.add(scrollerSpinner);
        controlPanel.add(pageCount);
        controlPanel.add(new JSeparator(JSeparator.VERTICAL));
        controlPanel.add(new MySeparator());
        controlPanel.add(portraitRBtn);
        controlPanel.add(landscapeRBtn);
        controlPanel.add(new MySeparator());
        controlPanel.add(new JLabel("Paper size:"));
        controlPanel.add(paperSizeCb);
        controlPanel.add(new MySeparator());
        controlPanel.add(backgroundsCb);
        add(controlPanel, BorderLayout.WEST);

        // Panel for close button
        JPanel closePanel = new JPanel();
        closePanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 7));
        closePanel.add(closeBtn);
        closePanel.setMinimumSize(new Dimension(105, 24));
        add(closePanel, BorderLayout.CENTER);
    }

    /**
     * We need enable/disable only controlPanel, close button should be enable
     * always
     */
    @Override
    public void setEnabled(boolean enabled)
    {
        printBtn.setEnabled(enabled);
        scrollerSpinner.setEnabled(enabled);
        pageCount.setEnabled(enabled);
        portraitRBtn.setEnabled(enabled);
        landscapeRBtn.setEnabled(enabled);
        paperSizeCb.setEnabled(enabled);
        backgroundsCb.setEnabled(enabled);
    }

    /**
     * Set a new print preview page count
     * 
     * @param count
     *            new count of pages
     */
    public void setPageCount(final int count)
    {
        // set maximum for scroller
        pageScroller.setMaximum(count);
        pageScroller.setValue(1);
        pageCount.setText("of " + count);
        this.repaint();
    }

    public ToolBarActionHandler<ActionEvent> getPrintBtnHandler()
    {
        return printBtnHandler;
    }

    public void setPrintBtnHandler(
            ToolBarActionHandler<ActionEvent> printBtnHandler)
    {
        this.printBtnHandler = printBtnHandler;
    }

    public ToolBarActionHandler<ChangeEvent> getScrollPageHandler()
    {
        return scrollPageHandler;
    }

    public void setScrollPageHandler(
            ToolBarActionHandler<ChangeEvent> scrollPageHandler)
    {
        this.scrollPageHandler = scrollPageHandler;
    }

    public ToolBarActionHandler<OrientationAction> getOrientationChangedHandler()
    {
        return orientationChangedHandler;
    }

    public void setOrientationChangedHandler(
            ToolBarActionHandler<OrientationAction> orientationChangedHandler)
    {
        this.orientationChangedHandler = orientationChangedHandler;
    }

    public ToolBarActionHandler<ActionEvent> getPaperSizeChangedHandler()
    {
        return paperSizeChangedHandler;
    }

    public void setPaperSizeChangedHandler(
            ToolBarActionHandler<ActionEvent> paperSizeChangedHandler)
    {
        this.paperSizeChangedHandler = paperSizeChangedHandler;
    }

    public ToolBarActionHandler<ActionEvent> getCloseBtnHandler()
    {
        return closeBtnHandler;
    }

    public void setCloseBtnHandler(
            ToolBarActionHandler<ActionEvent> closeBtnHandler)
    {
        this.closeBtnHandler = closeBtnHandler;
    }

    public ToolBarActionHandler<ActionEvent> getBackgroudsFlagChangedHandler()
    {
        return backgroudsFlagChangedHandler;
    }

    public void setBackgroudsFlagChangedHandler(
            ToolBarActionHandler<ActionEvent> backgroudsFlagChangedHandler)
    {
        this.backgroudsFlagChangedHandler = backgroudsFlagChangedHandler;
    }

}
