/*
 * Copyright (c) 1998-2008 TeamDev Ltd. All Rights Reserved.
 * Use is subject to license terms.
 */
package com.teamdev.xpcom.mozilla.demo.components;

import java.awt.Font;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextField;

import org.jdesktop.layout.GroupLayout;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.html.HTMLImageElement;

import com.teamdev.jxbrowser.event.ContextMenuEvent;
import com.teamdev.jxbrowser.event.ContextMenuListener;

public class PropertiesDialog extends JFrame
{
    public static final String LINK_PROPERTIES = "Link Properties";

    public static final String ADDRESS = "Address:";

    public static final String IMAGE_PROPERTIES = "Image Properties";

    public static final String WILL_OPEN_IN = "Will open in:";

    public static final String LOCATION = "Location:";

    public static final String ALTERNATE_TEXT = "Alternate text:";

    public static final String MISSING = "Missing";

    public static final String WIDTH = "Width:";

    public static final String HEIGHT = "Height:";

    public static final String SIZE_OF_FILE = "Size of File:";

    public static final String MISCELLANEOUS_PROPERTIES = "Miscellaneous Properties";

    public static final String SAME_WINDOW = "Same window";

    public static final String NEW_WINDOW = "New window";

    public static final String TITLE = "Title:";

    public static final int DEFAULT_WIDTH = 350;

    public static final int DEFAULT_HEIGHT = 150;

    private ContextMenuEvent _contextMenuEvent;

    private CloseKeyAdapter _closeKeyAdapter;

    public PropertiesDialog(ContextMenuEvent event)
    {
        _closeKeyAdapter = new CloseKeyAdapter();
        _contextMenuEvent = event;
        setTitle("Element properties");
        setIconImage(new ImageIcon(getClass().getResource("res/jxbrowser16x16.png")).getImage());
        setLocationRelativeTo(event.getWebBrowser().getComponent());

        PropertyPanel propertyPanel = new PropertyPanel();
        add(propertyPanel);

        setSize(DEFAULT_WIDTH, (int) (this.getPreferredSize().getHeight()) + 10);
    }

    private class CloseKeyAdapter extends KeyAdapter
    {
        public void keyPressed(KeyEvent e)
        {
            if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
            {
                dispose();
            }
        }
    }

    private class PropertyPanel extends JPanel
    {
        private boolean existLink;

        private String linkAddress;

        private String linkWillBeOpen;

        private String linkTitle;

        private boolean existPicture;

        private String pictureAddress;

        private String pictureWidth;

        private String pictureHeight;

        private String pictureAlternateText;

        private String pictureTitle;

        public PropertyPanel()
        {
            initContext();
            initParameters();
            initLayout();
        }

        private void initContext()
        {
            if (_contextMenuEvent.getContextFlags() == ContextMenuListener.CONTEXT_LINK)
            {
                existLink = true;
            }
            if (_contextMenuEvent.getContextFlags() == ContextMenuListener.CONTEXT_IMAGE)
            {
                existPicture = true;
            }
            if (_contextMenuEvent.getContextFlags() == ContextMenuListener.CONTEXT_LINK
                    + ContextMenuListener.CONTEXT_IMAGE)
            {
                existLink = true;
                existPicture = true;
            }
        }

        private void initParameters()
        {
            if (existLink)
            {
                Node firstNodeA = _contextMenuEvent.getTargetNode();
                for (int i = 0; i < 10; i++)
                {
                    if (firstNodeA.getNodeName().equals("A"))
                    {
                        break;
                    }
                    else
                    {
                        firstNodeA = firstNodeA.getParentNode();
                    }
                }

                NamedNodeMap nodeMap = firstNodeA.getAttributes();
                linkWillBeOpen = SAME_WINDOW;
                for (int i = 0; i < nodeMap.getLength(); i++)
                {
                    Node item = nodeMap.item(i);
                    if (item.getNodeName().equals("href") || item.getNodeName().equals("src"))
                    {
                        linkAddress = item.getNodeValue().trim();
                        if (!linkAddress.startsWith("http://"))
                        {
                            if (_contextMenuEvent.getAssociatedLink() != null)
                            {
                                linkAddress = _contextMenuEvent.getAssociatedLink();
                            }
                            else
                            {
                                linkAddress = "file://" + linkAddress;
                            }
                        }
                    }
                    else if (item.getNodeName().equals("target") && item.getNodeValue().equals("_blank"))
                    {
                        linkWillBeOpen = NEW_WINDOW;
                    }
                    else if (item.getNodeName().equals("title"))
                    {
                        linkTitle = item.getNodeValue().trim();
                    }
                }
            }
            if (existPicture)
            {
                HTMLImageElement imageElement = (HTMLImageElement) _contextMenuEvent.getTargetNode();
                if (imageElement != null)
                {
                    pictureAddress = imageElement.getSrc();
                    pictureWidth = imageElement.getWidth();
                    pictureHeight = imageElement.getHeight();
                    pictureAlternateText = imageElement.getAlt();
                    if (pictureAlternateText == null || pictureAlternateText.equals(""))
                    {
                        pictureAlternateText = MISSING;
                    }
                    pictureTitle = imageElement.getTitle();
                }
            }
        }

        private void initLayout()
        {
            GroupLayout layout = new GroupLayout(this);
            setLayout(layout);
            GroupLayout.SequentialGroup sGForHorizontal = layout.createSequentialGroup();
            GroupLayout.SequentialGroup sGForVertical = layout.createSequentialGroup();

            GroupLayout.ParallelGroup pGHorizontal = layout.createParallelGroup();
            GroupLayout.SequentialGroup sGVertical = layout.createSequentialGroup();

            if (existLink)
            {
                JLabel linkPropertiesLabel = new JLabel(LINK_PROPERTIES);
                JSeparator linkSeparator = new JSeparator(JSeparator.HORIZONTAL);
                JLabel linkAddressLabel = new JLabel(ADDRESS);
                JTextField linkAddressTextField = new JTextField(linkAddress);
                linkAddressTextField.setBorder(BorderFactory.createEmptyBorder());
                linkAddressTextField.setEditable(false);
                linkAddressTextField.setCaretPosition(0);
                linkAddressTextField.addKeyListener(_closeKeyAdapter);

                JLabel linkWillBeOpenLabel = new JLabel(WILL_OPEN_IN);
                JTextField linkWillBeOpenTextField = new JTextField(linkWillBeOpen);
                linkWillBeOpenTextField.setBorder(BorderFactory.createEmptyBorder());
                linkWillBeOpenTextField.setEditable(false);
                linkWillBeOpenTextField.setCaretPosition(0);
                linkWillBeOpenTextField.addKeyListener(_closeKeyAdapter);

                // gorizontal layout
                GroupLayout.SequentialGroup sequentialG1ForHorizontal = layout.createSequentialGroup();
                sequentialG1ForHorizontal.add(8);
                sequentialG1ForHorizontal.add(linkPropertiesLabel);

                GroupLayout.ParallelGroup parallelG1ForHorizontal = layout.createParallelGroup();
                parallelG1ForHorizontal.add(linkAddressLabel);
                parallelG1ForHorizontal.add(linkWillBeOpenLabel);

                GroupLayout.ParallelGroup parallelG2ForHorizontal = layout.createParallelGroup();
                parallelG2ForHorizontal.add(linkAddressTextField);
                parallelG2ForHorizontal.add(linkWillBeOpenTextField);

                GroupLayout.SequentialGroup sequentialG2ForHorizontal = layout.createSequentialGroup();
                sequentialG2ForHorizontal.add(20, 20, 20);
                sequentialG2ForHorizontal.add(parallelG1ForHorizontal);
                sequentialG2ForHorizontal.add(10, 10, 10);
                sequentialG2ForHorizontal.add(parallelG2ForHorizontal);
                sequentialG2ForHorizontal.add(10, 10, 10);

                // vertical layout
                GroupLayout.ParallelGroup parallelG1ForVertical = layout.createParallelGroup(GroupLayout.BASELINE);
                parallelG1ForVertical.add(linkAddressLabel);
                parallelG1ForVertical.add(linkAddressTextField);

                GroupLayout.ParallelGroup parallelG2ForVertical = layout.createParallelGroup(GroupLayout.BASELINE);
                parallelG2ForVertical.add(linkWillBeOpenLabel);
                parallelG2ForVertical.add(linkWillBeOpenTextField);

                GroupLayout.SequentialGroup sequentialG1ForVertical = layout.createSequentialGroup();
                sequentialG1ForVertical.add(10, 10, 10);
                sequentialG1ForVertical.add(parallelG1ForVertical);
                sequentialG1ForVertical.add(3, 3, 3);
                sequentialG1ForVertical.add(parallelG2ForVertical);
                sequentialG1ForVertical.add(20, 20, 20);

                pGHorizontal.add(sequentialG1ForHorizontal);
                pGHorizontal.add(linkSeparator);
                pGHorizontal.add(sequentialG2ForHorizontal);

                sGVertical.add(linkPropertiesLabel);
                sGVertical.add(5, 5, 5);
                sGVertical.add(linkSeparator, 2, 2, 2);
                sGVertical.add(sequentialG1ForVertical);
            }

            if (existPicture)
            {
                JLabel picturePropertiesLabel = new JLabel(IMAGE_PROPERTIES);
                JSeparator pictureSeparator = new JSeparator(JSeparator.HORIZONTAL);
                JLabel pictureAddressLabel = new JLabel(LOCATION);
                JTextField pictureAddressTextField = new JTextField(pictureAddress);
                pictureAddressTextField.setBorder(BorderFactory.createEmptyBorder());
                pictureAddressTextField.setEditable(false);
                pictureAddressTextField.setCaretPosition(0);
                pictureAddressTextField.addKeyListener(_closeKeyAdapter);
                JLabel pictureWidthLabel = new JLabel(PropertiesDialog.WIDTH);
                JTextField pictureWidthTextField = new JTextField(pictureWidth + " pixels");
                pictureWidthTextField.setBorder(BorderFactory.createEmptyBorder());
                pictureWidthTextField.setEditable(false);
                pictureWidthTextField.setCaretPosition(0);
                pictureWidthTextField.addKeyListener(_closeKeyAdapter);
                JLabel pictureHieghtLabel = new JLabel(PropertiesDialog.HEIGHT);
                JTextField pictureHieghtTextField = new JTextField(pictureHeight + " pixels");
                pictureHieghtTextField.setBorder(BorderFactory.createEmptyBorder());
                pictureHieghtTextField.setEditable(false);
                pictureHieghtTextField.setCaretPosition(0);
                pictureHieghtTextField.addKeyListener(_closeKeyAdapter);
                JLabel pictureAlternateTextLable = new JLabel(ALTERNATE_TEXT);
                JTextField pictureAlternateTextTextField = new JTextField(pictureAlternateText);
                pictureAlternateTextTextField.setBorder(BorderFactory.createEmptyBorder());
                pictureAlternateTextTextField.setEditable(false);
                pictureAlternateTextTextField.setCaretPosition(0);
                pictureAlternateTextTextField.addKeyListener(_closeKeyAdapter);
                if (pictureAlternateText.equals(MISSING))
                {
                    pictureAlternateTextTextField.setFont(new Font("Courier", Font.ITALIC, 11));
                }

                // gorizontal layout
                GroupLayout.SequentialGroup sequentialG1ForHorizontal = layout.createSequentialGroup();
                sequentialG1ForHorizontal.add(8);
                sequentialG1ForHorizontal.add(picturePropertiesLabel);

                GroupLayout.ParallelGroup parallelG1ForHorizontal = layout.createParallelGroup();
                parallelG1ForHorizontal.add(pictureAddressLabel);
                parallelG1ForHorizontal.add(pictureWidthLabel);
                parallelG1ForHorizontal.add(pictureHieghtLabel);
                parallelG1ForHorizontal.add(pictureAlternateTextLable);

                GroupLayout.ParallelGroup parallelG2ForHorizontal = layout.createParallelGroup();
                parallelG2ForHorizontal.add(pictureAddressTextField);
                parallelG2ForHorizontal.add(pictureWidthTextField);
                parallelG2ForHorizontal.add(pictureHieghtTextField);
                parallelG2ForHorizontal.add(pictureAlternateTextTextField);

                GroupLayout.SequentialGroup sequentialG2ForHorizontal = layout.createSequentialGroup();
                sequentialG2ForHorizontal.add(20, 20, 20);
                sequentialG2ForHorizontal.add(parallelG1ForHorizontal);
                sequentialG2ForHorizontal.add(10, 10, 10);
                sequentialG2ForHorizontal.add(parallelG2ForHorizontal);
                sequentialG2ForHorizontal.add(10, 10, 10);

                // vertical layout
                GroupLayout.ParallelGroup parallelG1ForVertical = layout.createParallelGroup(GroupLayout.BASELINE);
                parallelG1ForVertical.add(pictureAddressLabel);
                parallelG1ForVertical.add(pictureAddressTextField);

                GroupLayout.ParallelGroup parallelG2ForVertical = layout.createParallelGroup(GroupLayout.BASELINE);
                parallelG2ForVertical.add(pictureWidthLabel);
                parallelG2ForVertical.add(pictureWidthTextField);

                GroupLayout.ParallelGroup parallelG3ForVertical = layout.createParallelGroup(GroupLayout.BASELINE);
                parallelG3ForVertical.add(pictureHieghtLabel);
                parallelG3ForVertical.add(pictureHieghtTextField);

                GroupLayout.ParallelGroup parallelG4ForVertical = layout.createParallelGroup(GroupLayout.BASELINE);
                parallelG4ForVertical.add(pictureAlternateTextLable);
                parallelG4ForVertical.add(pictureAlternateTextTextField);

                GroupLayout.SequentialGroup sequentialG1ForVertical = layout.createSequentialGroup();
                sequentialG1ForVertical.add(10, 10, 10);
                sequentialG1ForVertical.add(parallelG1ForVertical);
                sequentialG1ForVertical.add(3, 3, 3);
                sequentialG1ForVertical.add(parallelG2ForVertical);
                sequentialG1ForVertical.add(3, 3, 3);
                sequentialG1ForVertical.add(parallelG3ForVertical);
                sequentialG1ForVertical.add(3, 3, 3);
                sequentialG1ForVertical.add(parallelG4ForVertical);
                sequentialG1ForVertical.add(20, 20, 20);

                pGHorizontal.add(sequentialG1ForHorizontal);
                pGHorizontal.add(pictureSeparator);
                pGHorizontal.add(sequentialG2ForHorizontal);

                sGVertical.add(picturePropertiesLabel);
                sGVertical.add(5, 5, 5);
                sGVertical.add(pictureSeparator, 2, 2, 2);
                sGVertical.add(sequentialG1ForVertical);
            }

            if ((linkTitle != null && !linkTitle.equals("")) || (pictureTitle != null && !pictureTitle.equals("")))
            {
                JLabel otherPropertiesLabel = new JLabel(MISCELLANEOUS_PROPERTIES);
                JSeparator otherSeparator = new JSeparator(JSeparator.HORIZONTAL);
                JLabel otherTitleLabel = new JLabel(TITLE);
                JTextField otherTitleTextField = new JTextField();
                if (((linkTitle != null && !linkTitle.equals("")) && (pictureTitle != null && !pictureTitle.equals("")))
                        || (pictureTitle != null && !pictureTitle.equals("")))
                {
                    otherTitleTextField.setText(pictureTitle);
                }
                else
                {
                    otherTitleTextField.setText(linkTitle);
                }
                otherTitleTextField.setBorder(BorderFactory.createEmptyBorder());
                otherTitleTextField.setEditable(false);
                otherTitleTextField.setCaretPosition(0);
                otherTitleTextField.addKeyListener(_closeKeyAdapter);

                // gorizontal layout
                GroupLayout.SequentialGroup sequentialG1ForHorizontal = layout.createSequentialGroup();
                sequentialG1ForHorizontal.add(8);
                sequentialG1ForHorizontal.add(otherPropertiesLabel);

                GroupLayout.ParallelGroup parallelG1ForHorizontal = layout.createParallelGroup();
                parallelG1ForHorizontal.add(otherTitleLabel);

                GroupLayout.ParallelGroup parallelG2ForHorizontal = layout.createParallelGroup();
                parallelG2ForHorizontal.add(otherTitleTextField);

                GroupLayout.SequentialGroup sequentialG2ForHorizontal = layout.createSequentialGroup();
                sequentialG2ForHorizontal.add(20, 20, 20);
                sequentialG2ForHorizontal.add(parallelG1ForHorizontal);
                sequentialG2ForHorizontal.add(10, 10, 10);
                sequentialG2ForHorizontal.add(parallelG2ForHorizontal);
                sequentialG2ForHorizontal.add(10, 10, 10);

                // vertical layout
                GroupLayout.ParallelGroup parallelG1ForVertical = layout.createParallelGroup(GroupLayout.BASELINE);
                parallelG1ForVertical.add(otherTitleLabel);
                parallelG1ForVertical.add(otherTitleTextField);

                GroupLayout.SequentialGroup sequentialG1ForVertical = layout.createSequentialGroup();
                sequentialG1ForVertical.add(10, 10, 10);
                sequentialG1ForVertical.add(parallelG1ForVertical);
                sequentialG1ForVertical.add(20, 20, 20);

                pGHorizontal.add(sequentialG1ForHorizontal);
                pGHorizontal.add(otherSeparator);
                pGHorizontal.add(sequentialG2ForHorizontal);

                sGVertical.add(otherPropertiesLabel);
                sGVertical.add(5, 5, 5);
                sGVertical.add(otherSeparator, 2, 2, 2);
                sGVertical.add(sequentialG1ForVertical);
            }

            sGForHorizontal.add(10, 10, 10);
            sGForHorizontal.add(pGHorizontal);
            sGForHorizontal.add(10, 10, 10);

            sGForVertical.add(10, 10, 10);
            sGForVertical.add(sGVertical);
            sGForVertical.add(10, 10, 10000);

            layout.setHorizontalGroup(sGForHorizontal);
            layout.setVerticalGroup(sGForVertical);
        }
    }

}
